% ==========================================================
% 
%           Neural Networks A Classroom Approach
%                     Satish Kumar
%             Copyright Tata McGraw Hill, 2004
%
%    MATLAB code that implements the SVM for non-linearly
%    separable data
%               Reference: Table 8.6;Page 294
%
% ==========================================================

X=[0.5 0.5
   0.5 1.0
   1.  1.5
   1.5 0.5
   1.5 2.0
   2.0 1.0
   2.5 2.0
   2.5 2.5];
 
D =[-1 -1 1 -1 -1 1 1 1]';
  
q = size(X,1);
C = 150;

% tolerance for Support Vector Detection
epsilon = C*1e-6;

ord = 2;					% Order of polynomial kernel
H = zeros(q,q);  		% Initialize Hessian matrix
for i=1:q				% Set up the Hessian
   for j=1:q
      H(i,j) = D(i)*D(j)*(X(i,:)*X(j,:)'+1)^ord; 
   end
end

f = -ones(n,1);  
    
% Add small amount of zero order regularisation to 
% avoid problems when Hessian is badly conditioned. 
H = H+1e-10*eye(size(H));
    
% Parameters for the Optimization problem
vlb = zeros(q,1);			% Lower bound of lambdas = 0
vub = C*ones(q,1);		% Upper bound C
x0 = zeros(q,1);			% Initial point is 0
numeqconstraints = 1;	% Number of equality constraints
A = D'; 						% Set up the equality constraint  
b = 0;        
 
% Solve the Optimization Problem
[alpha lambda how] = qp(H, f, A, b, vlb, vub, x0, numeqconstraints);
    
svindex = find( alpha > epsilon);
% Find unbounded support vectors
usvindex = find( alpha > epsilon & alpha < (C - epsilon)); 
if length(usvindex) > 0
  b0 =  (1/length(usvindex))*sum(D(usvindex) - H(usvindex,svindex)*alpha(svindex).*D(usvindex));
else 
  fprintf('No support vectors on margin - cannot compute bias.\n');
end

xmin = 0;
xmax = 3; 
ymin = 0;
ymax = 3; 
    
% Plot function value
[x,y] = meshgrid(xmin:(xmax-xmin)/50:xmax,ymin:(ymax-ymin)/50:ymax); 
z = b0*ones(size(x));
for x1 = 1 : size(x,1)
  for y1 = 1 : size(x,2) 
    input(1) = x(x1,y1);
    input(2) = y(x1,y1);
    for i = 1 : length(D)
      if (abs(alpha(i)) > epsilon)
        z(x1,y1) = z(x1,y1) + D(i)*alpha(i)*(input*X(i,:)'+ 1)^2; %Specific to poly!!!!
      end
    end      
  end
end
sp = pcolor(x,y,z);
shading interp;
colormap(bone);
    
hold on
for i = 1:size(D)
  if (D(i) == 1)
    h(1) = plot(X(i,1),X(i,2),'k^', 'MarkerSize', 8); % Class A
  else
    h(2) = plot(X(i,1),X(i,2),'w*', 'MarkerSize', 8); % Class B
  end
  if (abs(alpha(i)) > epsilon)
    plot(X(i,1),X(i,2),'wo', 'MarkerSize', 12) % Support Vector
  end
end 

% Plot Boundary contour

hold on
contour(x,y,z,[0 0],'w')
contour(x,y,z,[-1 -1],'w:')
contour(x,y,z,[1 1],'w:')
hold off
    
figure
mesh(x,y,z)
rotate3d on
hold on
decision = sign(z);
mesh(x,y,decision);
axis([0 3 0 3 -6 6]);
rotate3d on
colormap(bone)

    